/*!
\file skymap.h
\brief Class definition for Skymap
\author Nicholas Phillips, phillips@friday.gsfc.nasa.gov
\date July 17, 2003
*/

#ifndef SKYMAP_H
#define SKYMAP_H

#include <vector>
#include <string>
#include "chealpix.h"

//================================
//! What information is at each pixel
struct Pixel {
	float I;		//!< Intensity Stoke's parameter
	float Q;		//!< Q Stoke's parameter
	float U;		//!< U Stoke's parameter
	float Nobs;		//!< Number of observations
};

/*!
 Things I want this class to do:
   - Load and save from FITS files
   - Interface to HealPix 'C' functions
   
@author Nicholas Phillips
*/
class Skymap : public std::vector<Pixel> {
public:
	Skymap();
	Skymap(const int nside__,bool ring_order__ = true) { nside(nside__,ring_order__); }
	virtual ~Skymap();
	//! Return the current Healpix nside parameter
	int nside() const { return nside_; }
	//! Set this Skymap to a specified Healpix nside parameter
	int nside(const int nside__);
	//! Set this Skymap to a specified Healpix nside parameter
	int nside(const int nside__, bool ring_order_);
	//! Return the number of pixels, aliased to std::vector<Pixel>'s size()
	unsigned int npixels() const { return size(); }
	//! Return the ordering: \c true => ring order, \c false => nested
	bool ordering() const { return ring_order; }
	//! Load from a FITS file
	bool loadfitsfile(std::string filename);
	//! Save to a FITS file
	bool savefitsfile(std::string filename);

	//! Interface to Healpix code
	void ang2pix(double theta, double phi, long *ipix) const;
	//! Interface to Healpix code
	void pix2ang(long ipix, double *theta, double *phi) const;

	//! \c true if this map has valid Q/U Stokes parameters
	bool has_polarization() const { return has_polar; }
	bool has_Nobs() const { return has_Nobs_; }
private:
	long nside_;		//!< Healpix resolution
	long npixels_;		//!< Number of pixels, according to nside_
	bool ring_order;	//!< \c true if in RING order, else in NESTED
	bool has_polar;		//! \c true if this map has valid Q/U Stokes parameters
	bool has_Nobs_;		//! \c true if this map has valid Nobs field


	public: void zero_mean();
};

//=============================================
/*! This provides an easy way to use the Healpix code.
     The classes' value of \c nside_ is assumed and
     \c ring_order is used to determine which function to use.
*/
inline void Skymap::ang2pix(double theta, double phi, long *ipix) const {
	if( ring_order ) ang2pix_ring(nside_, theta, phi, ipix);
	else ang2pix_nest(nside_, theta, phi, ipix);
	return;
}

//=============================================
/*! This provides an easy way to use the Healpix code.
     The classes' value of \c nside_ is assumed and
     \c ring_order is used to determine which function to use.
*/
inline void Skymap::pix2ang(long ipix, double *theta, double *phi) const {
	if( ring_order ) pix2ang_ring(nside_, ipix, theta, phi);
	else pix2ang_nest(nside_, ipix, theta, phi);
	return;
}


#endif
